# -- Path setup --------------------------------------------------------------
import os
import sys

sys.path.append(os.path.abspath('..'))

import drb.drivers.zip

# -- Project information -----------------------------------------------------
project = 'ZIP driver for DRB'
copyright = '2021, GAEL Systems'
author = 'GAEL Systems Editor'

# The full version, including alpha/beta/rc tags
version = drb.drivers.zip.__version__
release = drb.drivers.zip.__version__

# -- General configuration ---------------------------------------------------
extensions = [
    'sphinx.ext.doctest',
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.intersphinx',
    'sphinx.ext.autosectionlabel',
]
templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# -- Options for HTML output --------------------------------------------------
html_theme = 'sphinx_rtd_theme'
html_show_sourcelink = False

# -- InterSphinx configuration ------------------------------------------------
intersphinx_mapping = {
    'python': ('https://docs.python.org/', None),
}
# -- Napoleon configuration ---------------------------------------------------
napoleon_use_ivar = True
napoleon_use_param = True
napoleon_use_rtype = True
# -- Autodoc configuration ----------------------------------------------------
autodoc_default_options = {
    'member-order': 'alphabetical',
    'undoc-members': False,
    'exclude-members': '__weakref__, __hash__, __slots__, __dict__,'
                       '__module__, __abstractmethods__, __init__'
}



import io
import os
import unittest
import zipfile
from pathlib import Path

from drb.drivers.file import DrbFileFactory
from drb.exceptions.core import DrbNotImplementationException

from drb.drivers.zip import DrbZipFactory

SENTINEL_1_ROOT = "sentinel-1"

SENTINEL_1_MANIFEST_FAKE = "manifest_safe"
SENTINEL_1_SUPPORT = "support"


class TestDrbZip(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    zip_ok1 = current_path / "files" / "data-ok.zip"

    zip_s1 = current_path / "files" / "sentinel-1.zip"

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbZipFactory().create(self.node_file)
        return self.node

    def test_has_impl(self):
        node = self.open_node(str(self.zip_s1))

        first_node = node[0]
        node_manifest = first_node[SENTINEL_1_MANIFEST_FAKE]
        self.assertTrue(node_manifest.has_impl(zipfile.ZipExtFile))
        self.assertTrue(node_manifest.has_impl(io.BufferedIOBase))
        node_support = first_node[SENTINEL_1_SUPPORT]
        self.assertFalse(node_support.has_impl(zipfile.ZipExtFile))
        self.assertFalse(node_support.has_impl(io.BufferedIOBase))

    def test_get_impl_exception(self):
        node = self.open_node(str(self.zip_s1))

        first_node = node[0]

        node_support = first_node[SENTINEL_1_SUPPORT]

        with self.assertRaises(DrbNotImplementationException):
            node_support.get_impl(io.BufferedIOBase)

        with self.assertRaises(DrbNotImplementationException):
            node_support.get_impl(zipfile.ZipExtFile)

    def test_get_impl(self):
        node = self.open_node(str(self.zip_s1))

        first_node = node[0]

        node_manifest = first_node[SENTINEL_1_MANIFEST_FAKE]
        self.assertIsNotNone(node_manifest.get_impl(zipfile.ZipExtFile))

        impl = node_manifest.get_impl(zipfile.ZipExtFile)

        self.assertIsInstance(impl, zipfile.ZipExtFile)
        self.assertIsInstance(impl, io.BufferedIOBase)

        impl.close()
        first_node.close()

    def test_close(self):
        node = self.open_node(str(self.zip_s1))
        first_node = node[0]
        first_node.close()

    def test_get_impl_read_line(self):
        node = self.open_node(str(self.zip_s1))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_manifest = first_node[SENTINEL_1_MANIFEST_FAKE]
        self.assertIsNotNone(node_manifest.get_impl(zipfile.ZipExtFile))

        impl = node_manifest.get_impl(zipfile.ZipExtFile)

        impl.readline()
        impl.readline()
        line3 = impl.readline()
        self.assertIn("<informationPackageMap>", str(line3))

        impl.close()

    def test_get_impl_read_buffer(self):
        node = self.open_node(str(self.zip_s1))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_manifest = first_node[SENTINEL_1_MANIFEST_FAKE]
        self.assertIsNotNone(node_manifest.get_impl(zipfile.ZipExtFile))

        impl = node_manifest.get_impl(io.BufferedIOBase)

        impl.seek(588 + 40)

        buffer = impl.read(30)
        self.assertIn("<informationPackageMap>", str(buffer))

        print(impl.name)
        print(impl)

        impl.close()

    def test_file_has_impl(self):
        node = self.open_node(str(self.zip_s1))

        self.assertTrue(node.has_impl(io.BufferedIOBase))
        self.assertFalse(node.has_impl(zipfile.ZipExtFile))

    def test_get_file_impl(self):
        node = self.open_node(str(self.zip_s1))
        impl = node.get_impl(io.BufferedIOBase)
        self.assertIsNotNone(impl)
        self.assertIsInstance(impl, io.BufferedIOBase)

        impl.close()

    def test_get_file_impl_exception(self):
        node = self.open_node(str(self.zip_s1))

        with self.assertRaises(DrbNotImplementationException):
            node.get_impl(zipfile.ZipExtFile)



import os
import unittest
import drb.topics.resolver as resolver
from pathlib import Path
from drb.drivers.file import DrbFileFactory
from drb.exceptions.core import DrbException
from drb.drivers.zip import DrbZipFactory
from drb.drivers.zip.zip import DrbZipAttributeNames

MY_DATA1_TXT = "mydata1.txt"
MY_DATA2_TXT = "mydata2.txt"
EMPTY_DIR = "empty_dir"
ROOT_DIR_DATA = "data"
MY_DATA_TXT = "mydata.txt"
SENTINEL_1_ROOT = "sentinel-1"


class TestDrbZip(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    zip_ok1 = current_path / "files" / "data-ok.zip"
    zip_ko1 = current_path / "files" / "data-nok.zip"

    zip_ok2 = current_path / "files" / "data-ok2.zip"
    zip_ko2 = current_path / "files" / "data-nok2.zip"
    zip_s1 = current_path / "files" / "sentinel-1.zip"
    zip_confused_names = current_path / "files" / "data-name-confused.zip"
    not_zip_files = current_path / "files" / "mydata.txt"

    zip_fake = current_path / "files" / "fake.zip"

    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbZipFactory().create(self.node_file)
        return self.node

    def test_opened__file_node(self):
        node = self.open_node(str(self.zip_ok1))
        self.assertIsNotNone(node[ROOT_DIR_DATA, None, 0])

    def test_fake(self):
        node = self.open_node(str(self.zip_fake))

        self.assertEqual(node.name, "fake.zip")

        with self.assertRaises(DrbException):
            len(node)

    def test_open_URL_ok1(self):
        node = self.open_node(str(self.zip_ok1))
        self.assertIsNotNone(node[ROOT_DIR_DATA, None, 0])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA, None, 2]
        self.assertIsNotNone(node[ROOT_DIR_DATA]
                             [MY_DATA_TXT])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][ROOT_DIR_DATA]
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][EMPTY_DIR]

        self.assertEqual(len(node), 1)
        self.assertEqual(
            len(node[ROOT_DIR_DATA]),
            1)

    def test_open_URL_ko1(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertEqual(len(node), 1)

        self.assertIsNotNone(node[ROOT_DIR_DATA])
        with self.assertRaises(KeyError):
            node[MY_DATA_TXT]
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][ROOT_DIR_DATA]
        self.assertIsNotNone(node[ROOT_DIR_DATA]
                             [MY_DATA_TXT])

    def test_open_URL_ok2(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertIsNotNone(node[ROOT_DIR_DATA])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA, None, 2]
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][MY_DATA_TXT])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][ROOT_DIR_DATA]

        self.assertIsNotNone(node[ROOT_DIR_DATA][EMPTY_DIR])
        self.assertIsNotNone(node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR, None, 2]

        self.assertEqual(len(node), 1)
        self.assertEqual(
            len(node[ROOT_DIR_DATA]),
            2)
        self.assertEqual(
            len(node[ROOT_DIR_DATA][EMPTY_DIR]),
            1)
        self.assertEqual(
            len(node[ROOT_DIR_DATA][MY_DATA_TXT]),
            0)

        self.assertEqual(
            len(node[ROOT_DIR_DATA][EMPTY_DIR][
                    EMPTY_DIR]),
            2)
        self.assertEqual(
            len(node[ROOT_DIR_DATA][EMPTY_DIR][
                    EMPTY_DIR]),
            2)
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR][MY_DATA1_TXT])
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR][MY_DATA2_TXT])

    def test_open_URL_ko2(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertIsNotNone(node[ROOT_DIR_DATA])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA, None, 2]
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][MY_DATA_TXT])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][ROOT_DIR_DATA]

        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR])
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR, None, 2]

        self.assertEqual(len(node), 1)
        self.assertEqual(
            len(node[ROOT_DIR_DATA]),
            2)
        self.assertEqual(
            len(node[ROOT_DIR_DATA][EMPTY_DIR]),
            1)
        self.assertEqual(
            len(node[ROOT_DIR_DATA][MY_DATA_TXT]),
            0)

        self.assertEqual(
            len(node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR]),
            2)
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR][MY_DATA1_TXT])
        self.assertIsNotNone(
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR][MY_DATA2_TXT])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA][EMPTY_DIR][EMPTY_DIR]["mydata3.txt"]

    def test_first_end_child(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertEqual(len(node), 1)
        self.assertIsNotNone(node[0])

        self.assertEqual(node[0].name, ROOT_DIR_DATA)
        self.assertEqual(node[-1].name, ROOT_DIR_DATA)

        self.assertEqual(node[0][0].name, EMPTY_DIR)
        self.assertEqual(node[0][-1].name, MY_DATA_TXT)

        with self.assertRaises(IndexError):
            node[0][-1][0]
        with self.assertRaises(IndexError):
            node[0][-1][-1]

        self.assertEqual(node[0][EMPTY_DIR][0].name,
                         EMPTY_DIR)
        self.assertEqual(node[0][EMPTY_DIR][-1].name,
                         EMPTY_DIR)

        self.assertEqual(node[0][0][-1][0].name, MY_DATA1_TXT)
        self.assertEqual(node[0][0][-1][-1].name, MY_DATA2_TXT)

    def test_get_named_child_without_occurrence(self):
        node = self.open_node(str(self.zip_ok2))

        occurrence = node[ROOT_DIR_DATA][EMPTY_DIR]
        self.assertEqual(occurrence.name, EMPTY_DIR)

    def test_has_child(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertTrue(node[0].has_child(EMPTY_DIR))
        self.assertFalse(node[0].has_child('nono'))

        self.assertTrue(node[0].has_child())
        self.assertTrue(
            node[0][EMPTY_DIR].has_child())
        self.assertTrue(
            node[0][EMPTY_DIR]
            [EMPTY_DIR].has_child())
        self.assertFalse(
            node[0][0][-1]
            [-1].has_child())

    def test_get_child_at(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertTrue(node[0].has_child())

        self.assertEqual(node[0][0].name, EMPTY_DIR)
        self.assertEqual(node[0][0][0][0].name, MY_DATA1_TXT)
        self.assertEqual(node[0][0][0][1].name, MY_DATA2_TXT)
        with self.assertRaises(IndexError):
            node[0][0][0][3]

        self.assertEqual(node[0][0][0]
                         [-1].name, MY_DATA2_TXT)

    def test_namespace_uri(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertTrue(node[0].has_child())
        self.assertIsNone(node[0].namespace_uri)

    def test_value_uri(self):
        node = self.open_node(str(self.zip_ok2))

        self.assertTrue(node[0].has_child())
        self.assertIsNone(node[0].value)

    def test_test_namespace_uri_file(self):
        node = self.open_node(str(self.zip_ok1))

        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)
        self.assertEqual(node.name, self.node_file.name)

    def test_value_file(self):
        node = self.open_node(str(self.zip_ok1))

        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)
        self.assertEqual(node.value, self.node_file.value)

    def test_attributes_zip_files(self):
        node = self.open_node(str(self.zip_s1))

        list_attributes = node.attributes.keys()
        self.assertIn(('mode', None), list_attributes)
        self.assertIn(('size', None), list_attributes)
        self.assertEqual(node.get_attribute('mode'), 'REGULAR')

    def test_attributes(self):
        node = self.open_node(str(self.zip_s1))

        self.assertIsNotNone(node[SENTINEL_1_ROOT].attributes)
        list_attributes = node[SENTINEL_1_ROOT].attributes
        self.assertIn((DrbZipAttributeNames.DIRECTORY.value, None),
                      list_attributes.keys())
        self.assertEqual(list_attributes[(DrbZipAttributeNames.DIRECTORY.value,
                                          None)], True)

        with self.assertRaises(KeyError):
            list_attributes[(DrbZipAttributeNames.DIRECTORY.value, 'toto')]

    def test_get_attribute(self):
        node = self.open_node(str(self.zip_s1))

        self.assertIsNotNone(node[SENTINEL_1_ROOT] @ 'directory')
        self.assertEqual(True, node[SENTINEL_1_ROOT] @ 'directory')

        child = node[SENTINEL_1_ROOT]["manifest.safe"]
        self.assertEqual(False, child @ 'directory')
        self.assertEqual(False, child.get_attribute('directory'))
        self.assertEqual(14945, child @ 'size')
        self.assertEqual(14945, child.get_attribute('size'))
        self.assertAlmostEqual(4.9, child @ 'ratio', delta=0.1)
        self.assertAlmostEqual(4.9, child.get_attribute('ratio'), delta=0.1)
        self.assertAlmostEqual(3014, child @ 'packed', delta=10)
        self.assertAlmostEqual(3014, child.get_attribute('packed'), delta=10)

        child = node[SENTINEL_1_ROOT]["measurement"]
        self.assertEqual(0, child @ 'size', 0)
        self.assertEqual(0, child.get_attribute('size'), 0)
        self.assertEqual(0, child @ 'ratio', 0)
        self.assertEqual(0, child.get_attribute('ratio'), 0)
        self.assertEqual(0, child @ 'packed', 0)
        self.assertEqual(0, child.get_attribute('packed'), 0)

    def test_open_name_confused(self):
        node = self.open_node(str(self.zip_confused_names))

        self.assertIsNotNone(node[ROOT_DIR_DATA])
        with self.assertRaises(KeyError):
            node[ROOT_DIR_DATA, None, 2]

        first_node = node[ROOT_DIR_DATA]
        self.assertEqual(len(first_node), 6)

        node_file = first_node[MY_DATA_TXT]
        self.assertFalse(node_file.has_child())
        self.assertEqual(len(node_file), 0)

        node_dir = first_node["mydata"]
        self.assertEqual(len(node_dir), 2)
        self.assertTrue(node_dir.has_child())

        node_dir = first_node["myd"]
        self.assertEqual(len(node_dir), 1)
        self.assertTrue(node_dir.has_child())

        node_dir = first_node["mydata_empty"]
        self.assertEqual(len(node_dir), 0)
        self.assertFalse(node_dir.has_child())

        node_file = first_node["my"]
        self.assertFalse(node_file.has_child())
        self.assertEqual(len(node_file), 0)

        node_file = first_node["mydata_txt"]
        self.assertFalse(node_file.has_child())
        self.assertEqual(len(node_file), 0)
        self.assertEqual(node_file.name, "mydata_txt")

    def test_parent(self):
        first_node = self.open_node(str(self.zip_ok2))

        self.assertEqual(first_node.parent, self.node_file.parent)
        self.assertEqual(first_node.value, self.node_file.value)

        first_child = first_node[0]
        self.assertEqual(first_child.parent, first_node)

    def test_path(self):
        first_node = self.open_node(str(self.zip_ok2))

        self.assertEqual(Path(first_node.path.name).as_posix(),
                         Path(self.zip_ok2).as_posix())

        first_child = first_node[0]
        path_child = Path(self.zip_ok2).joinpath(first_child.name).as_posix()
        self.assertEqual(str(first_child.path.name), path_child)

    def test_setitem(self):
        with self.assertRaises(NotImplementedError):
            parent = self.open_node(str(self.zip_s1))
            child = resolver.create(str(self.not_zip_files.absolute()))
            parent[None] = child

    def test_delitem(self):
        with self.assertRaises(NotImplementedError):
            node = self.open_node(str(self.zip_s1))[SENTINEL_1_ROOT]
            del node['manifest.safe']



import enum
import io
import zipfile
from typing import Any, List, Dict, Optional, Tuple
from zipfile import ZipExtFile, ZipInfo
from deprecated import deprecated

from drb.core import DrbNode, ParsedPath, DrbFactory
from drb.exceptions.core import DrbNotImplementationException
from drb.nodes.abstract_node import AbstractNode
from drb.topics.resolver import resolve_children
from drb.exceptions.zip import DrbZipNodeException


class DrbZipAttributeNames(enum.Enum):
    SIZE = 'size'
    """
    The size of the file in bytes.
    """
    DIRECTORY = 'directory'
    """
    A boolean that tell if the file is a directory.
    """
    RATIO = 'ratio'
    """
    the ratio between the size of the compressed file and the original file.
    """
    PACKED = 'packed'
    """
    The size of the compressed file in bytes.
    """


class DrbZipNode(AbstractNode):
    """
    This node is used to browse the content of a zip container.

    Parameters:
        parent (DrbNode): The zip container.
        zip_info (ZipInfo): Class with attributes describing
                            each file in the ZIP archive.

    """

    def __init__(self, parent: DrbNode, zip_info: ZipInfo):
        super().__init__()
        self._zip_info = zip_info
        if zip_info is not None:
            self.name = self.__retrieve_name(zip_info)
            if not zip_info.is_dir():
                self.add_impl(ZipExtFile, _open_zip_node_entry)
        self.__init_attributes_from_info(zip_info)
        self._parent: DrbNode = parent
        self._children: List[DrbNode] = None
        self._path = None

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    def __init_attributes_from_info(self, info: ZipInfo):
        if info is not None:
            self @= (DrbZipAttributeNames.DIRECTORY.value, info.is_dir())
            if hasattr(info, 'file_size'):
                self @= DrbZipAttributeNames.SIZE.value, info.file_size
            else:
                self @= DrbZipAttributeNames.SIZE.value, 0
            ratio = 0
            if hasattr(info, 'compress_size'):
                compress_size = info.compress_size
                if compress_size > 0:
                    ratio = info.file_size / info.compress_size
            else:
                compress_size = 0

            self @= (DrbZipAttributeNames.RATIO.value, ratio)
            self @= (DrbZipAttributeNames.PACKED.value, compress_size)

    @staticmethod
    def __retrieve_name(zip_info: ZipInfo) -> str:
        if zip_info.filename.endswith('/'):
            name = zip_info.filename[:-1]
        else:
            name = zip_info.filename
        if '/' in name:
            name = name[name.rindex('/') + 1:]
        return name

    def get_file_list(self):
        return self.parent.get_file_list()

    def _is_a_child(self, filename):
        if not filename.startswith(self._zip_info.filename):
            return False

        filename = filename[len(self._zip_info.filename):]

        if not filename:
            return False

        if not filename.startswith('/') and \
                not self._zip_info.filename.endswith('/'):
            return False

        # Either the name do not contains sep either only one a last position
        return '/' not in filename or filename.index('/') == (
                len(filename) - 1)

    @property
    @resolve_children
    @deprecated(version="1.2.0", reason="drb core deprecation since 2.1.0")
    def children(self) -> List[DrbNode]:
        if self._children is None:
            self._children = [DrbZipNode(self, entry) for entry in
                              self.get_file_list()
                              if self._is_a_child(entry.filename)]
            self._children = sorted(self._children,
                                    key=lambda entry_cmp: entry_cmp.name)

        return self._children

    def open_entry(self, zip_info: ZipInfo):
        # open the entry on zip file to return ZipExtFile
        # we back to the first node_file to open is
        return self.parent.open_entry(zip_info)


def _open_zip_node_entry(node: DrbZipNode, **kwargs) -> ZipExtFile:
    return node.open_entry(node._zip_info)


class DrbBaseZipNode(DrbZipNode):
    """
    This node is used to open a zip container, and browse his content.

    Parameters:
        base_node (DrbNode): The base node.
    """

    def __init__(self, base_node: DrbNode):
        super().__init__(parent=base_node.parent, zip_info=None)
        self._file_list = None
        self._zip_file = None
        self._zip_file_source = None
        self.base_node = base_node

    @property
    def parent(self) -> Optional[DrbNode]:
        """
        Returns the parent of the base node.

        Returns:
            DrbNode: the parent of the node
        """
        return self.base_node.parent

    @property
    def path(self) -> ParsedPath:
        """
        Returns the path of the base node.

        Returns:
            ParsedPath: the full path of the base node
        """
        return self.base_node.path

    @property
    def name(self) -> str:
        """
        Return the name of the base node.
        This name doesn't contain the path.

        Returns:
            str: the base node name
        """
        return self.base_node.name

    @property
    def value(self) -> Optional[Any]:
        """
        Return the value of the base node.

        Returns:
            Any: the value
        """
        return self.base_node.value

    @property
    def attributes(self) -> Dict[Tuple[str, str], Any]:
        """
        Return the attributes of the base node.

        Returns:
            Dict: Key(key_name, key_namespace): value(Any)
        """
        return self.base_node.attributes

    def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
        """
        Return a specific attributes of the base node.

        Parameters:
            name (str): The name of the attribute.
            namespace_uri (str): The namespace_uri of the attribute
                                 (default: None).
        Returns:
            Any: the attribute
        """
        return self.base_node.get_attribute(name, namespace_uri)

    @property
    def zip_file(self) -> zipfile.ZipFile:
        if self._zip_file is None:
            try:
                if self.base_node.has_impl(io.BufferedIOBase):
                    self._zip_file_source = self.base_node \
                        .get_impl(io.BufferedIOBase)
                    self._zip_file = zipfile.ZipFile(self._zip_file_source)
                else:
                    raise DrbZipNodeException(f'Unsupported base_node '
                                              f'{type(self.base_node).name} '
                                              f'for DrbFileZipNode')
            except Exception as e:
                raise DrbZipNodeException(f'Unable to read zip file'
                                          f' {self.name} ') from e
        return self._zip_file

    def has_impl(self, impl: type) -> bool:
        return self.base_node.has_impl(impl)

    def get_impl(self, impl: type, **kwargs) -> Any:
        if self.base_node.has_impl(impl):
            return self.base_node.get_impl(impl)
        raise DrbNotImplementationException

    def impl_capabilities(self) -> List[type]:
        return self.base_node.impl_capabilities()

    def close(self):
        if self._zip_file_source is not None:
            self._zip_file_source.close()
        if self._zip_file is not None:
            self._zip_file.close()
        self.base_node.close()

    def __add_dir_for_path(self, file_info):
        self._file_list.append(file_info)

        if file_info.filename[:-1].find('/') > 0:
            index = file_info.filename[:-1].rindex('/')
            if index > 0:
                path_zip = file_info.filename[:index + 1]
                if path_zip not in self.zip_file.NameToInfo.keys() and \
                        not any(
                            x.filename == path_zip for x in self._file_list):
                    self.__add_dir_for_path(
                        zipfile.ZipInfo(path_zip, file_info.date_time))

    def get_file_list(self):
        if self._file_list is None:
            self._file_list = []
            for fileInfo in self.zip_file.filelist:
                self.__add_dir_for_path(fileInfo)

        return self._file_list

    def _is_a_child(self, filename):
        if '/' not in filename or filename.index('/') == (len(filename) - 1):
            return True
        return False

    def open_entry(self, zip_info: zipfile.ZipInfo):
        # open a entry of the zip en return an BufferedIOBase impl
        return self._zip_file.open(zip_info)


class DrbZipFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        if isinstance(node, DrbBaseZipNode) or \
                isinstance(node, DrbZipNode):
            return node
        return DrbBaseZipNode(base_node=node)



from zipfile import ZipExtFile

from drb.drivers.file import DrbFileNode

from drb.drivers.zip import DrbBaseZipNode
from drb.drivers.zip.zip import DrbZipAttributeNames

path = 'path/to/a/container.zip'

baseNode = DrbFileNode(path)
zipNode = DrbBaseZipNode(baseNode)

# Access a children with his name
zipNode['subFile']
zipNode['subDirectory']['subFile']

# Check if a file has children
zipNode['subDirectory']['subFile'].has_child()

# Check if a specific child his present
zipNode.has_child(name='subDirectory')

# Get the first child
zipNode[0]

# Get all the attributes of the zip node
zipNode.attributes

# search for a specific attributes
zipNode.get_attribute(name=DrbZipAttributeNames.DIRECTORY.value)

# Read the content of a file in the container
zipNode.get_impl(ZipExtFile)
